/*
Produces a circle which passes through the two given points and has the given radius

Adapted by Aaron Parks from:

http://stackoverflow.com/questions/12264841/determine-circle-center-based-on-two-points-radius-known-with-solve-optim
*/
module circFromPoints(p1, p2, R, variation=0) {

  dy = p2[1] - p1[1];
  dx = p2[0] - p1[0];

  minrad = (1/2)*sqrt(pow(dx,2) + pow(dy,2));


  if(R < minrad){
    echo("ERROR: Specified radius can't be achieved with this data!!");
  }



/* 

I used a parametric equation to create the line going through the mean of the two points that is perpendicular to the line connecting the two points
(k) = ((x1+x2)/2, (y1+y2)/2) + k*(y2-y1, x1-x2) 
That is the vector equation for our line.  Then we can for any given value of k calculate the radius of the circle since we have the center and a value for a point on the edge of the circle.  Squaring the radius, subtracting R^2,and equating to 0 gives us the value of t to get a circle with the desired radius.  The following are the coefficients we get from doing that:
*/

  A = (pow(dy,2) + pow(dx,2));
  B = 0;
  C = (1/4)*(pow(dx,2) + pow(dy,2)) - pow(R,2);



/* We could just solve the quadratic equation but eh... polyroot is good enough
*/
//    k = as.numeric(polyroot(c(C, B, A)))
  k = [(B-sqrt(pow(B,2)-4*A*C))/(2*A), (B+sqrt(pow(B,2)-4*A*C))/(2*A)];
  


/*
Now we just plug our solution in to get the center of the circles that meet our specifications
*/
  mnX = (p1[0]+p2[0])/2; // Mean X
  mnY = (p1[1]+p2[1])/2; // Mean Y
  
  ansX1 = mnX + k[0]*dy;
  ansY1 = mnY + k[0]*(-dx);

  ansX2 = mnX + k[1]*dy;
  ansY2 = mnY + k[1]*(-dx);

/*
Decide which solution to actually draw
*/
  if(variation==0) {
    translate([ansX1, ansY1]) circle(R);
  } else {
    translate([ansX2, ansY2]) circle(R);
  }
}
