/*
  Generates a fluidic latch based on the Coanda effect. This device is based on illustrations in the Jun 1967 Popular Science article, "Fluidics: How they've taught a stream of air to think."
*/
use <circFromPoints.scad>

module coandaLatch(powerFeedLen=2) {

  powerFeedDiam=1.3;
  outFeedWidth=9;
  outFeedHeight=10;
  outFeedDiam=1.5;
  ctlFeedLen=3.5;
  ctlFeedDiam=0.5;
  coandaRadius=19;
  dividerIntrusion=6.75;
  ctlSpacer=0.75;


  union() {

    // Power feed

    translate([-0.5*powerFeedDiam,-powerFeedLen])  square([powerFeedDiam,powerFeedLen]);

    // Output feeds and dividing wall
    difference() {
    
      translate([-outFeedWidth/2,0]) 
	square([outFeedWidth,outFeedHeight]);

      circFromPoints([-0.5*powerFeedDiam,0],
		     [-outFeedWidth/2,outFeedHeight],
		     coandaRadius,
		     0);

      circFromPoints([0.5*powerFeedDiam,0],
		     [outFeedWidth/2,outFeedHeight],
		     coandaRadius,
		     1);

      polygon([[0,outFeedHeight-dividerIntrusion],
	       [-outFeedWidth/2+outFeedDiam,outFeedHeight+0.1], 
	       [outFeedWidth/2-outFeedDiam, outFeedHeight+0.1]]);

    }

    // Control feeds
    translate([0,outFeedHeight-dividerIntrusion-ctlFeedDiam-ctlSpacer]) {
      square([ctlFeedLen, ctlFeedDiam]);
    }

    mirror([1,0])
    translate([0,outFeedHeight-dividerIntrusion-ctlFeedDiam-ctlSpacer]) {
      square([ctlFeedLen, ctlFeedDiam]);
    }
  }
}
