from lga_experiment import Experiment
from lga_params import *
import random as random

CELLSIZE=39 # Must be ODD!
MAXR=11
MAXC=13

class ThreeParticleMacroGate(Experiment):
    def __init__(self, *args):
        Experiment.__init__(self, *args)
        self.name = ""

    def setup(self):
        self.setupEverything()
        self.visualizer.lgaChanged()

    def setupEverything(self):
        self.currentStatFn = self.stat_density
        self.loadBlankLGA((self.cg(MAXR)+2,self.cg(MAXC)+2))
        self.setup_boundaries()
        self.setup_particles()

        self.SRCVPARTICLES = WW


    def start(self):
        ENDTIME = 2000
        PRINTINTERVAL = 10

        self.setupEverything()

        nr, nc = self.lga.A.shape

        # Setup statistical regions
        self.lga.addRegion(1, self.cg(9), self.cg(0), self.cg(12), self.cg(3)) #inA
        self.lga.addRegion(2, self.cg(9), self.cg(8), self.cg(12), self.cg(11)) #inB
        self.lga.addRegion(3, self.cg(0), self.cg(4), self.cg(3), self.cg(7)) #inC
        self.lga.addRegion(4, self.cg(10), self.cg(4), self.cg(13), self.cg(7)) #NAND_Out
        self.lga.addRegion(5, self.cg(1), self.cg(0), self.cg(4), self.cg(3)) #NOT out (Top)
        self.lga.addRegion(5, self.cg(1), self.cg(8), self.cg(4), self.cg(11)) #NOT out (Bottom)



        self.visualizer.lgaChanged()

        print "Running until ", ENDTIME
        while self.lga.t < ENDTIME:
            self.step()
            if (self.lga.t % PRINTINTERVAL == 0):
                print self.lga.t

        myax = self.visualizer.ui_plot_stats('Particle density map. ') # Show graph of stats

        return


    def setup_particles(self):
        # Put in a bunch of random particles in a rectangular shape
        nr, nc = self.lga.A.shape
        #Print "Setting up particles L! nr=",nr, " nc=",nc

        inA = (self.cg(0)+3,self.cg(9)+3,self.cg(3)-3,self.cg(12))
        inB = (self.cg(8)+3,self.cg(9)+3,self.cg(11)-3,self.cg(12))
        inC = (self.cg(4)+3,self.cg(0)+3,self.cg(7)-3,self.cg(3))
        
        self.lga.RectNFill(0.1,inA)
        self.lga.RectNFill(0.1,inB)
        self.lga.RectNFill(0.1,inC)
 
        return


    def setup_boundaries(self):
        # Set up container / boundaries / world

        self.lga.DrawLine(self.cg(0), self.cg(1), self.cg(0), self.cg(4), REV)
        self.lga.DrawLine(self.cg(0), self.cg(4), self.cg(2), self.cg(4), REV)
        self.lga.DrawLine(self.cg(2), self.cg(4), self.cg(4), self.cg(6), REV)
        self.lga.DrawLine(self.cg(4), self.cg(6), self.cg(4), self.cg(7), REV)
        self.lga.DrawLine(self.cg(4), self.cg(7), self.cg(2), self.cg(9), REV)
        self.lga.DrawLine(self.cg(2), self.cg(9), self.cg(0), self.cg(9), REV)
        self.lga.DrawLine(self.cg(0), self.cg(9), self.cg(0), self.cg(12), REV)
        self.lga.DrawLine(self.cg(0), self.cg(12), self.cg(3), self.cg(12), REV)
        self.lga.DrawLine(self.cg(3), self.cg(12), self.cg(3), self.cg(10), REV)
        self.lga.DrawLine(self.cg(3), self.cg(10), self.cg(5), self.cg(8), REV)
        self.lga.DrawLine(self.cg(5), self.cg(8), self.cg(5), self.cg(10), REV)
        self.lga.DrawLine(self.cg(5), self.cg(10), self.cg(4), self.cg(10), REV)
        self.lga.DrawLine(self.cg(4), self.cg(10), self.cg(4), self.cg(13), REV)
        self.lga.DrawLine(self.cg(4), self.cg(13), self.cg(7), self.cg(13), REV)
        self.lga.DrawLine(self.cg(7), self.cg(13), self.cg(7), self.cg(10), REV)
        self.lga.DrawLine(self.cg(7), self.cg(10), self.cg(6), self.cg(10), REV)
        self.lga.DrawLine(self.cg(6), self.cg(10), self.cg(6), self.cg(8), REV)
        self.lga.DrawLine(self.cg(6), self.cg(8), self.cg(8), self.cg(10), REV)
        self.lga.DrawLine(self.cg(8), self.cg(10), self.cg(8), self.cg(12), REV)
        self.lga.DrawLine(self.cg(8), self.cg(12), self.cg(11), self.cg(12), REV)
        self.lga.DrawLine(self.cg(11), self.cg(12), self.cg(11), self.cg(9), REV)
        self.lga.DrawLine(self.cg(11), self.cg(9), self.cg(9), self.cg(9), REV)
        self.lga.DrawLine(self.cg(9), self.cg(9), self.cg(7), self.cg(7), REV)
        self.lga.DrawLine(self.cg(7), self.cg(7), self.cg(7), self.cg(6), REV)
        self.lga.DrawLine(self.cg(7), self.cg(6), self.cg(9), self.cg(4), REV)
        self.lga.DrawLine(self.cg(9), self.cg(4), self.cg(11), self.cg(4), REV)
        self.lga.DrawLine(self.cg(11), self.cg(4), self.cg(11), self.cg(1), REV)
        self.lga.DrawLine(self.cg(11), self.cg(1), self.cg(8), self.cg(1), REV)
        self.lga.DrawLine(self.cg(8), self.cg(1), self.cg(8), self.cg(3), REV)
        self.lga.DrawLine(self.cg(8), self.cg(3), self.cg(6), self.cg(5), REV)
        self.lga.DrawLine(self.cg(6), self.cg(5), self.cg(6), self.cg(3), REV)
        self.lga.DrawLine(self.cg(6), self.cg(3), self.cg(7), self.cg(3), REV)
        self.lga.DrawLine(self.cg(7), self.cg(3), self.cg(7), self.cg(0), REV)
        self.lga.DrawLine(self.cg(7), self.cg(0), self.cg(4), self.cg(0), REV)
        self.lga.DrawLine(self.cg(4), self.cg(0), self.cg(4), self.cg(3), REV)
        self.lga.DrawLine(self.cg(4), self.cg(3), self.cg(5), self.cg(3), REV)
        self.lga.DrawLine(self.cg(5), self.cg(3), self.cg(5), self.cg(5), REV)
        self.lga.DrawLine(self.cg(5), self.cg(5), self.cg(3), self.cg(3), REV)
        self.lga.DrawLine(self.cg(3), self.cg(3), self.cg(3), self.cg(1), REV)
        self.lga.DrawLine(self.cg(3), self.cg(1), self.cg(0), self.cg(1), REV)

        self.lga.DrawLine(self.cg(0)+0, self.cg(1)+1, self.cg(0)+0, self.cg(4)+1, REV)
        self.lga.DrawLine(self.cg(0)+0, self.cg(4)+1, self.cg(2)+0, self.cg(4)+1, REV)
        self.lga.DrawLine(self.cg(2)+1, self.cg(4)+0, self.cg(4)+1, self.cg(6)+0, REV)
        self.lga.DrawLine(self.cg(4)+1, self.cg(6)+0, self.cg(4)+1, self.cg(7)+0, REV)
        self.lga.DrawLine(self.cg(4)+1, self.cg(7)+0, self.cg(2)+1, self.cg(9)+0, REV)
        self.lga.DrawLine(self.cg(2)+0, self.cg(9)+1, self.cg(0)+0, self.cg(9)+1, REV)
        self.lga.DrawLine(self.cg(0)+1, self.cg(9)+0, self.cg(0)+1, self.cg(12)+0, REV)
        self.lga.DrawLine(self.cg(0)+0, self.cg(12)+1, self.cg(3)+0, self.cg(12)+1, REV)
        self.lga.DrawLine(self.cg(3)+1, self.cg(12)+0, self.cg(3)+1, self.cg(10)+0, REV)
        self.lga.DrawLine(self.cg(3)+1, self.cg(10)+0, self.cg(5)+1, self.cg(8)+0, REV)
        self.lga.DrawLine(self.cg(5)+1, self.cg(8)+0, self.cg(5)+1, self.cg(10)+0, REV)
        self.lga.DrawLine(self.cg(5)+0, self.cg(10)+1, self.cg(4)+0, self.cg(10)+1, REV)
        self.lga.DrawLine(self.cg(4)+1, self.cg(10)+0, self.cg(4)+1, self.cg(13)+0, REV)
        self.lga.DrawLine(self.cg(4)+0, self.cg(13)+1, self.cg(7)+0, self.cg(13)+1, REV)
        self.lga.DrawLine(self.cg(7)+1, self.cg(13)+0, self.cg(7)+1, self.cg(10)+0, REV)
        self.lga.DrawLine(self.cg(7)+0, self.cg(10)+1, self.cg(6)+0, self.cg(10)+1, REV)
        self.lga.DrawLine(self.cg(6)+1, self.cg(10)+0, self.cg(6)+1, self.cg(8)+0, REV)
        self.lga.DrawLine(self.cg(6)+1, self.cg(8)+0, self.cg(8)+1, self.cg(10)+0, REV)
        self.lga.DrawLine(self.cg(8)+1, self.cg(10)+0, self.cg(8)+1, self.cg(12)+0, REV)
        self.lga.DrawLine(self.cg(8)+0, self.cg(12)+1, self.cg(11)+0, self.cg(12)+1, REV)
        self.lga.DrawLine(self.cg(11)+1, self.cg(12)+0, self.cg(11)+1, self.cg(9)+0, REV)
        self.lga.DrawLine(self.cg(11)+0, self.cg(9)+1, self.cg(9)+0, self.cg(9)+1, REV)
        self.lga.DrawLine(self.cg(9)+1, self.cg(9)+0, self.cg(7)+1, self.cg(7)+0, REV)
        self.lga.DrawLine(self.cg(7)+1, self.cg(7)+0, self.cg(7)+1, self.cg(6)+0, REV)
        self.lga.DrawLine(self.cg(7)+1, self.cg(6)+0, self.cg(9)+1, self.cg(4)+0, REV)
        self.lga.DrawLine(self.cg(9)+0, self.cg(4)+1, self.cg(11)+0, self.cg(4)+1, REV)
        self.lga.DrawLine(self.cg(11)+1, self.cg(4)+0, self.cg(11)+1, self.cg(1)+0, REV)
        self.lga.DrawLine(self.cg(11)+0, self.cg(1)+1, self.cg(8)+0, self.cg(1)+1, REV)
        self.lga.DrawLine(self.cg(8)+1, self.cg(1)+0, self.cg(8)+1, self.cg(3)+0, REV)
        self.lga.DrawLine(self.cg(8)+1, self.cg(3)+0, self.cg(6)+1, self.cg(5)+0, REV)
        self.lga.DrawLine(self.cg(6)+1, self.cg(5)+0, self.cg(6)+1, self.cg(3)+0, REV)
        self.lga.DrawLine(self.cg(6)+0, self.cg(3)+1, self.cg(7)+0, self.cg(3)+1, REV)
        self.lga.DrawLine(self.cg(7)+1, self.cg(3)+0, self.cg(7)+1, self.cg(0)+0, REV)
        self.lga.DrawLine(self.cg(7)+0, self.cg(0)+1, self.cg(4)+0, self.cg(0)+1, REV)
        self.lga.DrawLine(self.cg(4)+1, self.cg(0)+0, self.cg(4)+1, self.cg(3)+0, REV)
        self.lga.DrawLine(self.cg(4)+0, self.cg(3)+1, self.cg(5)+0, self.cg(3)+1, REV)
        self.lga.DrawLine(self.cg(5)+1, self.cg(3)+0, self.cg(5)+1, self.cg(5)+0, REV)
        self.lga.DrawLine(self.cg(5)+1, self.cg(5)+0, self.cg(3)+1, self.cg(3)+0, REV)
        self.lga.DrawLine(self.cg(3)+1, self.cg(3)+0, self.cg(3)+1, self.cg(1)+0, REV)
        self.lga.DrawLine(self.cg(3)+0, self.cg(1)+1, self.cg(0)+0, self.cg(1)+1, REV)
        
        return
    

    def cg(self, index):
        return CELLSIZE*index
